@@ -9,8 +9,8 @@ import (
 	"strings"
 
 	"github.com/idursun/jjui/internal/ui/ace_jump"
-	"github.com/idursun/jjui/internal/ui/context"
 	"github.com/idursun/jjui/internal/ui/operations/duplicate"
+	"github.com/idursun/jjui/internal/ui/operations/megamerge"
 	"github.com/idursun/jjui/internal/ui/operations/revert"
 
 	"github.com/idursun/jjui/internal/parser"
@@ -35,29 +35,28 @@ import (
 )
 
 type Model struct {
-	rows              []parser.Row
-	tag               uint64
-	revisionToSelect  string
-	offScreenRows     []parser.Row
-	streamer          *graph.GraphStreamer
-	rowsChan          <-chan parser.RowBatch
-	controlChan       chan parser.ControlMsg
-	hasMore           bool
-	op                operations.Operation
-	cursor            int
-	width             int
-	height            int
-	context           *appContext.MainContext
-	keymap            config.KeyMappings[key.Binding]
-	output            string
-	err               error
-	aceJump           *ace_jump.AceJump
-	quickSearch       string
-	selectedRevisions map[string]bool
-	previousOpLogId   string
-	isLoading         bool
-	w                 *graph.Renderer
-	textStyle         lipgloss.Style
+	rows             []parser.Row
+	tag              uint64
+	revisionToSelect string
+	offScreenRows    []parser.Row
+	streamer         *graph.GraphStreamer
+	rowsChan         <-chan parser.RowBatch
+	controlChan      chan parser.ControlMsg
+	hasMore          bool
+	op               operations.Operation
+	cursor           int
+	width            int
+	height           int
+	context          *appContext.MainContext
+	keymap           config.KeyMappings[key.Binding]
+	output           string
+	err              error
+	aceJump          *ace_jump.AceJump
+	quickSearch      string
+	previousOpLogId  string
+	isLoading        bool
+	w                *graph.Renderer
+	textStyle        lipgloss.Style
 }
 
 type revisionsMsg struct {
@@ -140,11 +139,18 @@ func (m *Model) SelectedRevision() *jj.Commit {
 
 func (m *Model) SelectedRevisions() jj.SelectedRevisions {
 	var selected []*jj.Commit
+	ids := make(map[string]bool)
+	for _, ci := range m.context.CheckedItems {
+		if rev, ok := ci.(appContext.SelectedRevision); ok {
+			ids[rev.CommitId] = true
+		}
+	}
 	for _, row := range m.rows {
-		if m.selectedRevisions[row.Commit.GetChangeId()] {
+		if _, ok := ids[row.Commit.CommitId]; ok {
 			selected = append(selected, row.Commit)
 		}
 	}
+
 	if len(selected) == 0 {
 		return jj.NewSelectedRevisions(m.SelectedRevision())
 	}
@@ -185,8 +191,7 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 		}
 	case common.RefreshMsg:
 		if !msg.KeepSelections {
-			m.selectedRevisions = make(map[string]bool)
-			m.context.ClearCheckedItems(reflect.TypeFor[context.SelectedRevision]())
+			m.context.ClearCheckedItems(reflect.TypeFor[appContext.SelectedRevision]())
 		}
 		m.isLoading = true
 		cmd, _ := m.updateOperation(msg)
@@ -205,7 +210,17 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 	case startRowsStreamingMsg:
 		m.offScreenRows = nil
 		m.revisionToSelect = msg.selectedRevision
-		log.Println("Starting streaming revisions message received with tag:", msg.tag)
+
+		// If the revision to select is not set, use the currently selected item
+		if m.revisionToSelect == "" {
+			switch selected := m.context.SelectedItem.(type) {
+			case appContext.SelectedRevision:
+				m.revisionToSelect = selected.CommitId
+			case appContext.SelectedFile:
+				m.revisionToSelect = selected.CommitId
+			}
+		}
+		log.Println("Starting streaming revisions message received with tag:", msg.tag, "revision to select:", msg.selectedRevision)
 		return m, m.requestMoreRows(msg.tag)
 	case appendRowsBatchMsg:
 		if msg.tag != m.tag {
@@ -297,14 +312,8 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 			case key.Matches(msg, m.keymap.ToggleSelect):
 				commit := m.rows[m.cursor].Commit
 				changeId := commit.GetChangeId()
-				isChecked := !m.selectedRevisions[changeId]
-				m.selectedRevisions[changeId] = isChecked
-				item := context.SelectedRevision{ChangeId: changeId, CommitId: commit.CommitId}
-				if isChecked {
-					m.context.AddCheckedItem(item)
-				} else {
-					m.context.RemoveCheckedItem(item)
-				}
+				item := appContext.SelectedRevision{ChangeId: changeId, CommitId: commit.CommitId}
+				m.context.ToggleCheckedItem(item)
 				immediate, _ := m.context.RunCommandImmediate(jj.GetParent(jj.NewSelectedRevisions(commit)))
 				parentIndex := m.selectRevision(string(immediate))
 				if parentIndex != -1 {
@@ -371,6 +380,8 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 				m.op = rebase.NewOperation(m.context, m.SelectedRevisions(), rebase.SourceRevision, rebase.TargetDestination)
 			case key.Matches(msg, m.keymap.Duplicate.Mode):
 				m.op = duplicate.NewOperation(m.context, m.SelectedRevisions(), duplicate.TargetDestination)
+			case key.Matches(msg, m.keymap.Megamerge):
+				m.op = megamerge.NewModel(m.context, m.SelectedRevision())
 			}
 		}
 	}
@@ -454,10 +465,9 @@ func (m *Model) View() string {
 		return lipgloss.Place(m.width, m.height, lipgloss.Center, lipgloss.Center, "(no matching revisions)")
 	}
 
-	renderer := graph.NewDefaultRowIterator(m.rows, graph.WithWidth(m.width), graph.WithStylePrefix("revisions"))
+	renderer := graph.NewDefaultRowIterator(m.rows, graph.WithWidth(m.width), graph.WithStylePrefix("revisions"), graph.WithSelections(m.context.GetSelectedRevisions()))
 	renderer.Op = m.op
 	renderer.Cursor = m.cursor
-	renderer.Selections = m.selectedRevisions
 	renderer.SearchText = m.quickSearch
 	renderer.AceJumpPrefix = m.aceJump.Prefix()
 
@@ -512,7 +522,7 @@ func (m *Model) loadStreaming(revset string, selectedRevision string, tag uint64
 	m.hasMore = true
 	m.offScreenRows = nil
 	log.Println("Starting streaming revisions with tag:", tag)
-	var startStreamingCmd = func() tea.Msg {
+	startStreamingCmd := func() tea.Msg {
 		return startRowsStreamingMsg{selectedRevision, tag}
 	}
 
@@ -579,17 +589,16 @@ func New(c *appContext.MainContext) Model {
 	keymap := config.Current.GetKeyMap()
 	w := graph.NewRenderer(20, 10)
 	return Model{
-		context:           c,
-		w:                 w,
-		keymap:            keymap,
-		rows:              nil,
-		offScreenRows:     nil,
-		op:                operations.NewDefault(),
-		cursor:            0,
-		width:             20,
-		height:            10,
-		selectedRevisions: make(map[string]bool),
-		textStyle:         common.DefaultPalette.Get("revisions text"),
+		context:       c,
+		w:             w,
+		keymap:        keymap,
+		rows:          nil,
+		offScreenRows: nil,
+		op:            operations.NewDefault(),
+		cursor:        0,
+		width:         20,
+		height:        10,
+		textStyle:     common.DefaultPalette.Get("revisions text"),
 	}
 }
 
